import pandas as pd
import pyarrow.parquet as pq
import glob
import numpy as np
import os
import io

#models
#linear
from sklearn.base import clone
from sklearn.linear_model import Lasso

from matplotlib import pyplot
from pandas.plotting import lag_plot
from pandas.plotting import autocorrelation_plot
from pandas import read_csv
from pandas.plotting import autocorrelation_plot
from statsmodels.tsa.ar_model import AutoReg
from statsmodels.tsa.stattools import adfuller
from statsmodels.graphics.tsaplots import plot_pacf
import pmdarima as pm
from math import sqrt


from numpy import mean
from numpy import std
from numpy import absolute
from numpy import arange

from scipy import stats
from scipy.stats import spearmanr
from scipy.stats import pearsonr
import pmdarima as pmd
import matplotlib.pyplot as plt
import pybats
from pybats.loss_functions import MAPE
from pybats.analysis import analysis
from pybats.point_forecast import median

from sklearn.model_selection import cross_val_score
from sklearn.metrics import accuracy_score
from sklearn.model_selection import RepeatedKFold
from sklearn.model_selection import GridSearchCV
from sklearn.linear_model import Lasso
from sklearn.metrics import mean_squared_error, mean_absolute_error, r2_score
import warnings
warnings.simplefilter(action='ignore', category=FutureWarning)

desired_width = 320
pd.set_option('display.width', desired_width)
pd.set_option('display.max_columns', 10)
pd.options.mode.chained_assignment = None

#Loading the datasets
#Loading all unemployment files
filesUnemployment = glob.glob('unemployment_data_us/adjusted/*')
filesUnemployment.sort()

filesUnemploymentCountry = glob.glob('unemployment_usa.csv')
filesUnemploymentCountry.sort()

#Loading all US states files
files = glob.glob('total_grouped_job_numbers.parquet')
# files = glob.glob('../data_bg_us_v1/us/*/*.parquet')
files.sort()

filesUSA = glob.glob('total_grouped_jobs_country.parquet')
filesUSA.sort()
fileUSA=filesUSA[0]

#LOOP COUNT
resultsDf = pd.DataFrame()
for unFile in filesUnemployment[:]:
    tempArray = []
    csvSample = pd.read_csv(unFile, skiprows=0, usecols = [i for i in range(2)])
    csvSample.columns=['date', 'unrate']

    #Getting the name of the state
    fileName_absolute = os.path.basename(unFile)
    tempArray.append(fileName_absolute[:2])
    #Dropping last month (May 2021)
    csvSample.drop(csvSample.tail(1).index, inplace=True)

    # Changing type to date
    csvSample.loc[:, 'date'] = pd.to_datetime(csvSample.loc[:, 'date'], format='%Y-%m-%d')

    #Changing type to float
    csvSample.loc[:, 'unrate'] = csvSample.loc[:, 'unrate'].astype(float)

    #Accessing data for a particular state
    jobsGrupedDf = pd.read_parquet(files[0])
    selectedColsDf = jobsGrupedDf.loc[jobsGrupedDf['CanonState'] == fileName_absolute[:2]]
    selectedColsDf.loc[:, 'JobDate'] = pd.to_datetime(selectedColsDf.loc[:,'JobDate'])

    #Merging relevant column
    selectedColsDf.loc[:, 'Unrate'] = csvSample.loc[:,'unrate'].values
    selectedColsDf.loc[:, 'Unrate'] = selectedColsDf.loc[:,'Unrate'].astype(float)
    print(' ')
    print('STATE', fileName_absolute[:2])

    #BAYESIAN
    selectedColsDf.set_index('JobDate')

    # Differencing data
    csvSample['unrate']=csvSample['unrate'].diff(1)

    corr, _ = spearmanr(selectedColsDf.Unrate, selectedColsDf.JobID)
    print('Spearmans correlation: %.3f' % corr)
    tempArray.append(format(corr, '.3f'))
    newSeries = pd.Series(tempArray)
    resultsDf = resultsDf.append(newSeries, ignore_index = True)

print(resultsDf)

#COUNTRY COUNT
print('')
print("COUNTRY")
csvSample = pd.read_csv(filesUnemploymentCountry[0], skiprows=0, usecols=[i for i in range(2)])
csvSample.columns = ['date', 'unrate']

# Dropping last month (May 2021)
csvSample.drop(csvSample.tail(1).index, inplace=True)

# Changing type to date
csvSample.loc[:, 'date'] = pd.to_datetime(csvSample.loc[:, 'date'], format='%Y-%m-%d')
usaDf = pd.read_parquet(fileUSA)
usaDf.loc[:, 'JobDate'] = pd.to_datetime(usaDf.loc[:, 'JobDate'])

# Merging relevant column
usaDf.loc[:, 'Unrate'] = csvSample.loc[:, 'unrate'].values
usaDf.loc[:, 'Unrate'] = usaDf.loc[:, 'Unrate'].astype(float)

# Differencing data
csvSample['unrate']=csvSample['unrate'].diff(1)
csvSample.drop(csvSample.head(1).index, inplace=True)
csvSample.index = pd.DatetimeIndex(csvSample.index).to_period('M')

corr, _ = spearmanr(usaDf.Unrate, usaDf.JobID)
print('Spearmans correlation: %.3f' % corr)

tempArray = []
tempArray.append('USA')
tempArray.append(format(corr, '.3f'))
newSeries = pd.Series(tempArray)
resultsDf = resultsDf.append(newSeries, ignore_index = True)

resultsDf.columns=['State', 'Spearmans correlation']
print(resultsDf)
resultsDf.to_csv('spearmans_correlation_all_states_and_usa.csv')